*** Settings ***
Documentation       Check that you can create an entity with one or more scopes

Resource            ${EXECDIR}/resources/ApiUtils/ContextInformationProvision.resource
Resource            ${EXECDIR}/resources/ApiUtils/ContextInformationConsumption.resource
Resource            ${EXECDIR}/resources/AssertionUtils.resource
Resource            ${EXECDIR}/resources/JsonUtils.resource

Test Teardown       Delete Initial Entity
Test Template       Create an entity with scopes


*** Variables ***
${building_id_prefix}=      urn:ngsi-ld:Building:


*** Test Cases ***    FILENAME    CONTENT_TYPE
001_10_01 EntityWithOneScope
    building-minimal-with-one-scope.json    application/json
001_10_02 EntityWithManyScopes
    building-minimal-with-many-scopes.json    application/json


*** Keywords ***
Create an entity with scopes
    [Documentation]    Check that you can create an entity with one or more scopes
    [Tags]    e-create    6_3_5    4_18    since_v1.5.1
    [Arguments]    ${filename}    ${content_type}

    ${entity_id}=    Generate Random Entity Id    ${building_id_prefix}
    ${response}=    Create Entity Selecting Content Type
    ...    ${filename}
    ...    ${entity_id}
    ...    ${content_type}
    Set Test Variable    ${entity_id}

    Check Response Status Code    201    ${response.status_code}
    Check Response Headers Containing URI set to    ${entity_id}    ${response.headers}
    ${created_entity}=    Load Test Sample    entities/${filename}    ${entity_id}
    IF    '${content_type}'=='application/json'
        ${response1}=    Retrieve Entity by Id    ${entity_id}    ${content_type}
    END
    IF    '${content_type}'=='application/ld+json'
        ${response1}=    Retrieve Entity by Id
        ...    ${entity_id}
        ...    ${content_type}
        ...    context=${ngsild_test_suite_context}
    END
    Check Created Resource Set To    ${created_entity}    ${response1.json()}

Delete Initial Entity
    Delete Entity by Id    ${entity_id}
